@@ -34,11 +34,11 @@
  *		plan normally and pass back the results.
  *
  *
- * Portions Copyright (c) 1996-2008, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2010, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/backend/executor/nodeResult.c,v 1.42 2008/01/01 19:45:49 momjian Exp $
+ *	  $PostgreSQL: pgsql/src/backend/executor/nodeResult.c,v 1.45 2010/01/02 16:57:45 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -230,8 +230,6 @@ ExecInitResult(Result *node, EState *estate, int eflags)
 
 	resstate->ps.ps_TupFromTlist = false;
 
-#define RESULT_NSLOTS 1
-
 	/*
 	 * tuple table initialization
 	 */
@@ -268,12 +266,6 @@ ExecInitResult(Result *node, EState *estate, int eflags)
 	return resstate;
 }
 
-int
-ExecCountSlotsResult(Result *node)
-{
-	return ExecCountSlotsNode(outerPlan(node)) + RESULT_NSLOTS;
-}
-
 /* ----------------------------------------------------------------
  *		ExecEndResult
  *
