@@ -164,16 +164,8 @@ WalSndHandshake(void)
 		int			firstchar;
 
 		/* Wait for a command to arrive */
-		pq_startmsgread();
 		firstchar = pq_getbyte();
 
-		/* Read the message contents */
-		if (firstchar != EOF)
-		{
-			if (pq_getmessage(&input_message, 0))
-				firstchar = EOF;	/* suitable message already logged */
-		}
-
 		/*
 		 * Emergency bailout if postmaster has died.  This is to avoid the
 		 * necessity for manual cleanup of all postmaster children.
@@ -191,6 +183,16 @@ WalSndHandshake(void)
 			ProcessConfigFile(PGC_SIGHUP);
 		}
 
+		if (firstchar != EOF)
+		{
+			/*
+			 * Read the message contents. This is expected to be done without
+			 * blocking because we've been able to get message type code.
+			 */
+			if (pq_getmessage(&input_message, 0))
+				firstchar = EOF;	/* suitable message already logged */
+		}
+
 		/* Handle the very limited subset of commands expected in this phase */
 		switch (firstchar)
 		{
@@ -329,7 +331,6 @@ CheckClosedConnection(void)
 	unsigned char firstchar;
 	int			r;
 
-	pq_startmsgread();
 	r = pq_getbyte_if_available(&firstchar);
 	if (r < 0)
 	{
@@ -342,7 +343,6 @@ CheckClosedConnection(void)
 	if (r == 0)
 	{
 		/* no data available without blocking */
-		pq_endmsgread();
 		return;
 	}
 
@@ -818,7 +818,7 @@ WalSndQuickDieHandler(SIGNAL_ARGS)
 	on_exit_reset();
 
 	/*
-	 * Note we do exit(2) not exit(0).  This is to force the postmaster into a
+	 * Note we do exit(2) not exit(0).	This is to force the postmaster into a
 	 * system reset cycle if some idiot DBA sends a manual SIGQUIT to a random
 	 * backend.  This is necessary precisely because we don't clean up our
 	 * shared memory state.  (The "dead man switch" mechanism in pmsignal.c
