@@ -32,11 +32,11 @@
  * entry, since that may need to change as a consequence of ALTER TABLE.
  *
  *
- * Portions Copyright (c) 1996-2006, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2010, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/backend/utils/cache/typcache.c,v 1.22.2.1 2010/09/02 03:17:12 tgl Exp $
+ *	  $PostgreSQL: pgsql/src/backend/utils/cache/typcache.c,v 1.32.6.1 2010/09/02 03:16:52 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -50,6 +50,7 @@
 #include "utils/builtins.h"
 #include "utils/inval.h"
 #include "utils/lsyscache.h"
+#include "utils/rel.h"
 #include "utils/syscache.h"
 #include "utils/typcache.h"
 
@@ -111,9 +112,6 @@ lookup_type_cache(Oid type_id, int flags)
 		/* First time through: initialize the hash table */
 		HASHCTL		ctl;
 
-		if (!CacheMemoryContext)
-			CreateCacheMemoryContext();
-
 		MemSet(&ctl, 0, sizeof(ctl));
 		ctl.keysize = sizeof(Oid);
 		ctl.entrysize = sizeof(TypeCacheEntry);
@@ -123,6 +121,10 @@ lookup_type_cache(Oid type_id, int flags)
 
 		/* Also set up a callback for relcache SI invalidations */
 		CacheRegisterRelcacheCallback(TypeCacheRelCallback, (Datum) 0);
+
+		/* Also make sure CacheMemoryContext exists */
+		if (!CacheMemoryContext)
+			CreateCacheMemoryContext();
 	}
 
 	/* Try to look up an existing entry */
@@ -139,9 +141,7 @@ lookup_type_cache(Oid type_id, int flags)
 		HeapTuple	tp;
 		Form_pg_type typtup;
 
-		tp = SearchSysCache(TYPEOID,
-							ObjectIdGetDatum(type_id),
-							0, 0, 0);
+		tp = SearchSysCache1(TYPEOID, ObjectIdGetDatum(type_id));
 		if (!HeapTupleIsValid(tp))
 			elog(ERROR, "cache lookup failed for type %u", type_id);
 		typtup = (Form_pg_type) GETSTRUCT(tp);
@@ -171,17 +171,30 @@ lookup_type_cache(Oid type_id, int flags)
 	/* If we haven't already found the opclass, try to do so */
 	if ((flags & (TYPECACHE_EQ_OPR | TYPECACHE_LT_OPR | TYPECACHE_GT_OPR |
 				  TYPECACHE_CMP_PROC |
-				  TYPECACHE_EQ_OPR_FINFO | TYPECACHE_CMP_PROC_FINFO)) &&
-		typentry->btree_opc == InvalidOid)
+				  TYPECACHE_EQ_OPR_FINFO | TYPECACHE_CMP_PROC_FINFO |
+				  TYPECACHE_BTREE_OPFAMILY)) &&
+		typentry->btree_opf == InvalidOid)
 	{
-		typentry->btree_opc = GetDefaultOpClass(type_id,
-												BTREE_AM_OID);
-		/* Only care about hash opclass if no btree opclass... */
-		if (typentry->btree_opc == InvalidOid)
+		Oid			opclass;
+
+		opclass = GetDefaultOpClass(type_id, BTREE_AM_OID);
+		if (OidIsValid(opclass))
 		{
-			if (typentry->hash_opc == InvalidOid)
-				typentry->hash_opc = GetDefaultOpClass(type_id,
-													   HASH_AM_OID);
+			typentry->btree_opf = get_opclass_family(opclass);
+			typentry->btree_opintype = get_opclass_input_type(opclass);
+		}
+		/* Only care about hash opclass if no btree opclass... */
+		if (typentry->btree_opf == InvalidOid)
+		{
+			if (typentry->hash_opf == InvalidOid)
+			{
+				opclass = GetDefaultOpClass(type_id, HASH_AM_OID);
+				if (OidIsValid(opclass))
+				{
+					typentry->hash_opf = get_opclass_family(opclass);
+					typentry->hash_opintype = get_opclass_input_type(opclass);
+				}
+			}
 		}
 		else
 		{
@@ -199,45 +212,50 @@ lookup_type_cache(Oid type_id, int flags)
 	if ((flags & (TYPECACHE_EQ_OPR | TYPECACHE_EQ_OPR_FINFO)) &&
 		typentry->eq_opr == InvalidOid)
 	{
-		if (typentry->btree_opc != InvalidOid)
-			typentry->eq_opr = get_opclass_member(typentry->btree_opc,
-												  InvalidOid,
-												  BTEqualStrategyNumber);
+		if (typentry->btree_opf != InvalidOid)
+			typentry->eq_opr = get_opfamily_member(typentry->btree_opf,
+												   typentry->btree_opintype,
+												   typentry->btree_opintype,
+												   BTEqualStrategyNumber);
 		if (typentry->eq_opr == InvalidOid &&
-			typentry->hash_opc != InvalidOid)
-			typentry->eq_opr = get_opclass_member(typentry->hash_opc,
-												  InvalidOid,
-												  HTEqualStrategyNumber);
+			typentry->hash_opf != InvalidOid)
+			typentry->eq_opr = get_opfamily_member(typentry->hash_opf,
+												   typentry->hash_opintype,
+												   typentry->hash_opintype,
+												   HTEqualStrategyNumber);
 	}
 	if ((flags & TYPECACHE_LT_OPR) && typentry->lt_opr == InvalidOid)
 	{
-		if (typentry->btree_opc != InvalidOid)
-			typentry->lt_opr = get_opclass_member(typentry->btree_opc,
-												  InvalidOid,
-												  BTLessStrategyNumber);
+		if (typentry->btree_opf != InvalidOid)
+			typentry->lt_opr = get_opfamily_member(typentry->btree_opf,
+												   typentry->btree_opintype,
+												   typentry->btree_opintype,
+												   BTLessStrategyNumber);
 	}
 	if ((flags & TYPECACHE_GT_OPR) && typentry->gt_opr == InvalidOid)
 	{
-		if (typentry->btree_opc != InvalidOid)
-			typentry->gt_opr = get_opclass_member(typentry->btree_opc,
-												  InvalidOid,
-												  BTGreaterStrategyNumber);
+		if (typentry->btree_opf != InvalidOid)
+			typentry->gt_opr = get_opfamily_member(typentry->btree_opf,
+												   typentry->btree_opintype,
+												   typentry->btree_opintype,
+												   BTGreaterStrategyNumber);
 	}
 	if ((flags & (TYPECACHE_CMP_PROC | TYPECACHE_CMP_PROC_FINFO)) &&
 		typentry->cmp_proc == InvalidOid)
 	{
-		if (typentry->btree_opc != InvalidOid)
-			typentry->cmp_proc = get_opclass_proc(typentry->btree_opc,
-												  InvalidOid,
-												  BTORDER_PROC);
+		if (typentry->btree_opf != InvalidOid)
+			typentry->cmp_proc = get_opfamily_proc(typentry->btree_opf,
+												   typentry->btree_opintype,
+												   typentry->btree_opintype,
+												   BTORDER_PROC);
 	}
 
 	/*
 	 * Set up fmgr lookup info as requested
 	 *
 	 * Note: we tell fmgr the finfo structures live in CacheMemoryContext,
-	 * which is not quite right (they're really in DynaHashContext) but this
-	 * will do for our purposes.
+	 * which is not quite right (they're really in the hash table's private
+	 * memory context) but this will do for our purposes.
 	 */
 	if ((flags & TYPECACHE_EQ_OPR_FINFO) &&
 		typentry->eq_opr_finfo.fn_oid == InvalidOid &&
@@ -263,7 +281,7 @@ lookup_type_cache(Oid type_id, int flags)
 	 */
 	if ((flags & TYPECACHE_TUPDESC) &&
 		typentry->tupDesc == NULL &&
-		typentry->typtype == 'c')
+		typentry->typtype == TYPTYPE_COMPOSITE)
 	{
 		Relation	rel;
 
@@ -411,15 +429,16 @@ assign_record_type_typmod(TupleDesc tupDesc)
 		/* First time through: initialize the hash table */
 		HASHCTL		ctl;
 
-		if (!CacheMemoryContext)
-			CreateCacheMemoryContext();
-
 		MemSet(&ctl, 0, sizeof(ctl));
 		ctl.keysize = REC_HASH_KEYS * sizeof(Oid);
 		ctl.entrysize = sizeof(RecordCacheEntry);
 		ctl.hash = tag_hash;
 		RecordCacheHash = hash_create("Record information cache", 64,
 									  &ctl, HASH_ELEM | HASH_FUNCTION);
+
+		/* Also make sure CacheMemoryContext exists */
+		if (!CacheMemoryContext)
+			CreateCacheMemoryContext();
 	}
 
 	/* Find or create a hashtable entry for this hash class */
