@@ -2,11 +2,11 @@
  *
  *	  UHC <--> UTF8
  *
- * Portions Copyright (c) 1996-2006, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2010, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/backend/utils/mb/conversion_procs/utf8_and_uhc/utf8_and_uhc.c,v 1.14.2.1 2009/01/29 19:24:19 tgl Exp $
+ *	  $PostgreSQL: pgsql/src/backend/utils/mb/conversion_procs/utf8_and_uhc/utf8_and_uhc.c,v 1.20 2010/01/02 16:57:57 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -44,8 +44,8 @@ uhc_to_utf8(PG_FUNCTION_ARGS)
 
 	CHECK_ENCODING_CONVERSION_ARGS(PG_UHC, PG_UTF8);
 
-	LocalToUtf(src, dest, LUmapUHC,
-			   sizeof(LUmapUHC) / sizeof(pg_local_to_utf), PG_UHC, len);
+	LocalToUtf(src, dest, LUmapUHC, NULL,
+			   sizeof(LUmapUHC) / sizeof(pg_local_to_utf), 0, PG_UHC, len);
 
 	PG_RETURN_VOID();
 }
@@ -59,8 +59,8 @@ utf8_to_uhc(PG_FUNCTION_ARGS)
 
 	CHECK_ENCODING_CONVERSION_ARGS(PG_UTF8, PG_UHC);
 
-	UtfToLocal(src, dest, ULmapUHC,
-			   sizeof(ULmapUHC) / sizeof(pg_utf_to_local), PG_UHC, len);
+	UtfToLocal(src, dest, ULmapUHC, NULL,
+			   sizeof(ULmapUHC) / sizeof(pg_utf_to_local), 0, PG_UHC, len);
 
 	PG_RETURN_VOID();
 }
