@@ -2,10 +2,10 @@
  *
  * vacuumdb
  *
- * Portions Copyright (c) 1996-2008, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2010, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
- * $PostgreSQL: pgsql/src/bin/scripts/vacuumdb.c,v 1.20 2008/01/01 19:45:56 momjian Exp $
+ * $PostgreSQL: pgsql/src/bin/scripts/vacuumdb.c,v 1.38 2010/06/11 23:58:24 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -14,14 +14,15 @@
 #include "common.h"
 
 
-static void vacuum_one_database(const char *dbname, bool full, bool verbose, bool analyze,
-					const char *table,
-					const char *host, const char *port,
-					const char *username, bool password,
+static void vacuum_one_database(const char *dbname, bool full, bool verbose,
+					bool and_analyze, bool analyze_only, bool freeze,
+					const char *table, const char *host, const char *port,
+					const char *username, enum trivalue prompt_password,
 					const char *progname, bool echo);
-static void vacuum_all_databases(bool full, bool verbose, bool analyze,
+static void vacuum_all_databases(bool full, bool verbose, bool and_analyze,
+					 bool analyze_only, bool freeze,
 					 const char *host, const char *port,
-					 const char *username, bool password,
+					 const char *username, enum trivalue prompt_password,
 					 const char *progname, bool echo, bool quiet);
 
 static void help(const char *progname);
@@ -34,11 +35,14 @@ main(int argc, char *argv[])
 		{"host", required_argument, NULL, 'h'},
 		{"port", required_argument, NULL, 'p'},
 		{"username", required_argument, NULL, 'U'},
+		{"no-password", no_argument, NULL, 'w'},
 		{"password", no_argument, NULL, 'W'},
 		{"echo", no_argument, NULL, 'e'},
 		{"quiet", no_argument, NULL, 'q'},
 		{"dbname", required_argument, NULL, 'd'},
 		{"analyze", no_argument, NULL, 'z'},
+		{"analyze-only", no_argument, NULL, 'Z'},
+		{"freeze", no_argument, NULL, 'F'},
 		{"all", no_argument, NULL, 'a'},
 		{"table", required_argument, NULL, 't'},
 		{"full", no_argument, NULL, 'f'},
@@ -54,21 +58,23 @@ main(int argc, char *argv[])
 	char	   *host = NULL;
 	char	   *port = NULL;
 	char	   *username = NULL;
-	bool		password = false;
+	enum trivalue prompt_password = TRI_DEFAULT;
 	bool		echo = false;
 	bool		quiet = false;
-	bool		analyze = false;
+	bool		and_analyze = false;
+	bool		analyze_only = false;
+	bool		freeze = false;
 	bool		alldb = false;
 	char	   *table = NULL;
 	bool		full = false;
 	bool		verbose = false;
 
 	progname = get_progname(argv[0]);
-	set_pglocale_pgservice(argv[0], "pgscripts");
+	set_pglocale_pgservice(argv[0], PG_TEXTDOMAIN("pgscripts"));
 
 	handle_help_version_opts(argc, argv, "vacuumdb", help);
 
-	while ((c = getopt_long(argc, argv, "h:p:U:Weqd:zat:fv", long_options, &optindex)) != -1)
+	while ((c = getopt_long(argc, argv, "h:p:U:wWeqd:zZFat:fv", long_options, &optindex)) != -1)
 	{
 		switch (c)
 		{
@@ -81,8 +87,11 @@ main(int argc, char *argv[])
 			case 'U':
 				username = optarg;
 				break;
+			case 'w':
+				prompt_password = TRI_NO;
+				break;
 			case 'W':
-				password = true;
+				prompt_password = TRI_YES;
 				break;
 			case 'e':
 				echo = true;
@@ -94,7 +103,13 @@ main(int argc, char *argv[])
 				dbname = optarg;
 				break;
 			case 'z':
-				analyze = true;
+				and_analyze = true;
+				break;
+			case 'Z':
+				analyze_only = true;
+				break;
+			case 'F':
+				freeze = true;
 				break;
 			case 'a':
 				alldb = true;
@@ -128,6 +143,23 @@ main(int argc, char *argv[])
 			exit(1);
 	}
 
+	if (analyze_only)
+	{
+		if (full)
+		{
+			fprintf(stderr, _("%s: cannot use the \"full\" option when performing only analyze\n"),
+					progname);
+			exit(1);
+		}
+		if (freeze)
+		{
+			fprintf(stderr, _("%s: cannot use the \"freeze\" option when performing only analyze\n"),
+					progname);
+			exit(1);
+		}
+		/* allow 'and_analyze' with 'analyze_only' */
+	}
+
 	setup_cancel_handler();
 
 	if (alldb)
@@ -145,8 +177,8 @@ main(int argc, char *argv[])
 			exit(1);
 		}
 
-		vacuum_all_databases(full, verbose, analyze,
-							 host, port, username, password,
+		vacuum_all_databases(full, verbose, and_analyze, analyze_only, freeze,
+							 host, port, username, prompt_password,
 							 progname, echo, quiet);
 	}
 	else
@@ -161,8 +193,9 @@ main(int argc, char *argv[])
 				dbname = get_user_name(progname);
 		}
 
-		vacuum_one_database(dbname, full, verbose, analyze, table,
-							host, port, username, password,
+		vacuum_one_database(dbname, full, verbose, and_analyze, analyze_only,
+							freeze, table,
+							host, port, username, prompt_password,
 							progname, echo);
 	}
 
@@ -171,10 +204,10 @@ main(int argc, char *argv[])
 
 
 static void
-vacuum_one_database(const char *dbname, bool full, bool verbose, bool analyze,
-					const char *table,
+vacuum_one_database(const char *dbname, bool full, bool verbose, bool and_analyze,
+					bool analyze_only, bool freeze, const char *table,
 					const char *host, const char *port,
-					const char *username, bool password,
+					const char *username, enum trivalue prompt_password,
 					const char *progname, bool echo)
 {
 	PQExpBufferData sql;
@@ -183,18 +216,62 @@ vacuum_one_database(const char *dbname, bool full, bool verbose, bool analyze,
 
 	initPQExpBuffer(&sql);
 
-	appendPQExpBuffer(&sql, "VACUUM");
-	if (full)
-		appendPQExpBuffer(&sql, " FULL");
-	if (verbose)
-		appendPQExpBuffer(&sql, " VERBOSE");
-	if (analyze)
-		appendPQExpBuffer(&sql, " ANALYZE");
+	conn = connectDatabase(dbname, host, port, username, prompt_password, progname);
+
+	if (analyze_only)
+	{
+		appendPQExpBuffer(&sql, "ANALYZE");
+		if (verbose)
+			appendPQExpBuffer(&sql, " VERBOSE");
+	}
+	else
+	{
+		appendPQExpBuffer(&sql, "VACUUM");
+		if (PQserverVersion(conn) >= 90000)
+		{
+			const char *paren = " (";
+			const char *comma = ", ";
+			const char *sep = paren;
+
+			if (full)
+			{
+				appendPQExpBuffer(&sql, "%sFULL", sep);
+				sep = comma;
+			}
+			if (freeze)
+			{
+				appendPQExpBuffer(&sql, "%sFREEZE", sep);
+				sep = comma;
+			}
+			if (verbose)
+			{
+				appendPQExpBuffer(&sql, "%sVERBOSE", sep);
+				sep = comma;
+			}
+			if (and_analyze)
+			{
+				appendPQExpBuffer(&sql, "%sANALYZE", sep);
+				sep = comma;
+			}
+			if (sep != paren)
+				appendPQExpBuffer(&sql, ")");
+		}
+		else
+		{
+			if (full)
+				appendPQExpBuffer(&sql, " FULL");
+			if (freeze)
+				appendPQExpBuffer(&sql, " FREEZE");
+			if (verbose)
+				appendPQExpBuffer(&sql, " VERBOSE");
+			if (and_analyze)
+				appendPQExpBuffer(&sql, " ANALYZE");
+		}
+	}
 	if (table)
 		appendPQExpBuffer(&sql, " %s", table);
 	appendPQExpBuffer(&sql, ";\n");
 
-	conn = connectDatabase(dbname, host, port, username, password, progname);
 	if (!executeMaintenanceCommand(conn, sql.data, echo))
 	{
 		if (table)
@@ -212,16 +289,16 @@ vacuum_one_database(const char *dbname, bool full, bool verbose, bool analyze,
 
 
 static void
-vacuum_all_databases(bool full, bool verbose, bool analyze,
-					 const char *host, const char *port,
-					 const char *username, bool password,
+vacuum_all_databases(bool full, bool verbose, bool and_analyze, bool analyze_only,
+					 bool freeze, const char *host, const char *port,
+					 const char *username, enum trivalue prompt_password,
 					 const char *progname, bool echo, bool quiet)
 {
 	PGconn	   *conn;
 	PGresult   *result;
 	int			i;
 
-	conn = connectDatabase("postgres", host, port, username, password, progname);
+	conn = connectDatabase("postgres", host, port, username, prompt_password, progname);
 	result = executeQuery(conn, "SELECT datname FROM pg_database WHERE datallowconn ORDER BY 1;", progname, echo);
 	PQfinish(conn);
 
@@ -235,8 +312,8 @@ vacuum_all_databases(bool full, bool verbose, bool analyze,
 			fflush(stdout);
 		}
 
-		vacuum_one_database(dbname, full, verbose, analyze, NULL,
-							host, port, username, password,
+		vacuum_one_database(dbname, full, verbose, and_analyze, analyze_only,
+						 freeze, NULL, host, port, username, prompt_password,
 							progname, echo);
 	}
 
@@ -253,18 +330,21 @@ help(const char *progname)
 	printf(_("\nOptions:\n"));
 	printf(_("  -a, --all                       vacuum all databases\n"));
 	printf(_("  -d, --dbname=DBNAME             database to vacuum\n"));
-	printf(_("  -t, --table='TABLE[(COLUMNS)]'  vacuum specific table only\n"));
-	printf(_("  -f, --full                      do full vacuuming\n"));
-	printf(_("  -z, --analyze                   update optimizer hints\n"));
 	printf(_("  -e, --echo                      show the commands being sent to the server\n"));
+	printf(_("  -f, --full                      do full vacuuming\n"));
+	printf(_("  -F, --freeze                    freeze row transaction information\n"));
 	printf(_("  -q, --quiet                     don't write any messages\n"));
+	printf(_("  -t, --table='TABLE[(COLUMNS)]'  vacuum specific table only\n"));
 	printf(_("  -v, --verbose                   write a lot of output\n"));
+	printf(_("  -z, --analyze                   update optimizer statistics\n"));
+	printf(_("  -Z, --analyze-only              only update optimizer statistics\n"));
 	printf(_("  --help                          show this help, then exit\n"));
 	printf(_("  --version                       output version information, then exit\n"));
 	printf(_("\nConnection options:\n"));
 	printf(_("  -h, --host=HOSTNAME       database server host or socket directory\n"));
 	printf(_("  -p, --port=PORT           database server port\n"));
 	printf(_("  -U, --username=USERNAME   user name to connect as\n"));
+	printf(_("  -w, --no-password         never prompt for password\n"));
 	printf(_("  -W, --password            force password prompt\n"));
 	printf(_("\nRead the description of the SQL command VACUUM for details.\n"));
 	printf(_("\nReport bugs to <pgsql-bugs@postgresql.org>.\n"));
