@@ -1,13 +1,11 @@
-/* $PostgreSQL: pgsql/src/interfaces/ecpg/ecpglib/misc.c,v 1.26.2.1 2006/06/21 11:38:26 meskes Exp $ */
+/* $PostgreSQL: pgsql/src/interfaces/ecpg/ecpglib/misc.c,v 1.59 2010/07/06 19:19:00 momjian Exp $ */
 
 #define POSTGRES_ECPG_INTERNAL
 #include "postgres_fe.h"
 
 #include <limits.h>
 #include <unistd.h>
-#ifdef ENABLE_THREAD_SAFETY
-#include <pthread.h>
-#endif
+#include "ecpg-pthread-win32.h"
 #include "ecpgtype.h"
 #include "ecpglib.h"
 #include "ecpgerrno.h"
@@ -17,16 +15,19 @@
 #include "pgtypes_date.h"
 #include "pgtypes_timestamp.h"
 #include "pgtypes_interval.h"
+#include "pg_config_paths.h"
 
-#ifdef HAVE_LONG_LONG_INT_64
+#ifdef HAVE_LONG_LONG_INT
 #ifndef LONG_LONG_MIN
 #ifdef LLONG_MIN
 #define LONG_LONG_MIN LLONG_MIN
 #else
 #define LONG_LONG_MIN LONGLONG_MIN
-#endif
-#endif
-#endif
+#endif   /* LLONG_MIN */
+#endif   /* LONG_LONG_MIN */
+#endif   /* HAVE_LONG_LONG_INT */
+
+bool		ecpg_internal_regression_mode = false;
 
 static struct sqlca_t sqlca_init =
 {
@@ -95,21 +96,21 @@ static int	simple_debug = 0;
 static FILE *debugstream = NULL;
 
 void
-ECPGinit_sqlca(struct sqlca_t * sqlca)
+ecpg_init_sqlca(struct sqlca_t * sqlca)
 {
 	memcpy((char *) sqlca, (char *) &sqlca_init, sizeof(struct sqlca_t));
 }
 
 bool
-ECPGinit(const struct connection * con, const char *connection_name, const int lineno)
+ecpg_init(const struct connection * con, const char *connection_name, const int lineno)
 {
 	struct sqlca_t *sqlca = ECPGget_sqlca();
 
-	ECPGinit_sqlca(sqlca);
+	ecpg_init_sqlca(sqlca);
 	if (con == NULL)
 	{
-		ECPGraise(lineno, ECPG_NO_CONN, ECPG_SQLSTATE_CONNECTION_DOES_NOT_EXIST,
-				  connection_name ? connection_name : "NULL");
+		ecpg_raise(lineno, ECPG_NO_CONN, ECPG_SQLSTATE_CONNECTION_DOES_NOT_EXIST,
+				   connection_name ? connection_name : ecpg_gettext("NULL"));
 		return (false);
 	}
 
@@ -120,8 +121,7 @@ ECPGinit(const struct connection * con, const char *connection_name, const int l
 static void
 ecpg_sqlca_key_destructor(void *arg)
 {
-	if (arg != NULL)
-		free(arg);				/* sqlca structure allocated in ECPGget_sqlca */
+	free(arg);					/* sqlca structure allocated in ECPGget_sqlca */
 }
 
 static void
@@ -143,7 +143,7 @@ ECPGget_sqlca(void)
 	if (sqlca == NULL)
 	{
 		sqlca = malloc(sizeof(struct sqlca_t));
-		ECPGinit_sqlca(sqlca);
+		ecpg_init_sqlca(sqlca);
 		pthread_setspecific(sqlca_key, sqlca);
 	}
 	return (sqlca);
@@ -155,31 +155,47 @@ ECPGget_sqlca(void)
 bool
 ECPGstatus(int lineno, const char *connection_name)
 {
-	struct connection *con = ECPGget_connection(connection_name);
+	struct connection *con = ecpg_get_connection(connection_name);
 
-	if (!ECPGinit(con, connection_name, lineno))
+	if (!ecpg_init(con, connection_name, lineno))
 		return (false);
 
 	/* are we connected? */
 	if (con->connection == NULL)
 	{
-		ECPGraise(lineno, ECPG_NOT_CONN, ECPG_SQLSTATE_ECPG_INTERNAL_ERROR, con->name);
+		ecpg_raise(lineno, ECPG_NOT_CONN, ECPG_SQLSTATE_ECPG_INTERNAL_ERROR, con->name);
 		return false;
 	}
 
 	return (true);
 }
 
+PGTransactionStatusType
+ECPGtransactionStatus(const char *connection_name)
+{
+	const struct connection *con;
+
+	con = ecpg_get_connection(connection_name);
+	if (con == NULL)
+	{
+		/* transaction status is unknown */
+		return PQTRANS_UNKNOWN;
+	}
+
+	return PQtransactionStatus(con->connection);
+
+}
+
 bool
 ECPGtrans(int lineno, const char *connection_name, const char *transaction)
 {
 	PGresult   *res;
-	struct connection *con = ECPGget_connection(connection_name);
+	struct connection *con = ecpg_get_connection(connection_name);
 
-	if (!ECPGinit(con, connection_name, lineno))
+	if (!ecpg_init(con, connection_name, lineno))
 		return (false);
 
-	ECPGlog("ECPGtrans line %d action = %s connection = %s\n", lineno, transaction, con ? con->name : "(nil)");
+	ecpg_log("ECPGtrans on line %d: action \"%s\"; connection \"%s\"\n", lineno, transaction, con ? con->name : "null");
 
 	/* if we have no connection we just simulate the command */
 	if (con && con->connection)
@@ -193,27 +209,18 @@ ECPGtrans(int lineno, const char *connection_name, const char *transaction)
 		if (con->committed && !con->autocommit && strncmp(transaction, "begin", 5) != 0 && strncmp(transaction, "start", 5) != 0)
 		{
 			res = PQexec(con->connection, "begin transaction");
-			if (res == NULL || PQresultStatus(res) != PGRES_COMMAND_OK)
-			{
-				ECPGraise(lineno, ECPG_TRANS, ECPG_SQLSTATE_TRANSACTION_RESOLUTION_UNKNOWN, NULL);
+			if (!ecpg_check_PQresult(res, lineno, con->connection, ECPG_COMPAT_PGSQL))
 				return FALSE;
-			}
 			PQclear(res);
 		}
 
 		res = PQexec(con->connection, transaction);
-		if (res == NULL || PQresultStatus(res) != PGRES_COMMAND_OK)
-		{
-			ECPGraise(lineno, ECPG_TRANS, ECPG_SQLSTATE_TRANSACTION_RESOLUTION_UNKNOWN, NULL);
+		if (!ecpg_check_PQresult(res, lineno, con->connection, ECPG_COMPAT_PGSQL))
 			return FALSE;
-		}
 		PQclear(res);
-	}
 
-	if (strcmp(transaction, "commit") == 0 || strcmp(transaction, "rollback") == 0)
-		con->committed = true;
-	else
-		con->committed = false;
+		con->committed = (PQtransactionStatus(con->connection) == PQTRANS_IDLE);
+	}
 
 	return true;
 }
@@ -226,9 +233,17 @@ ECPGdebug(int n, FILE *dbgs)
 	pthread_mutex_lock(&debug_init_mutex);
 #endif
 
-	simple_debug = n;
+	if (n > 100)
+	{
+		ecpg_internal_regression_mode = true;
+		simple_debug = n - 100;
+	}
+	else
+		simple_debug = n;
+
 	debugstream = dbgs;
-	ECPGlog("ECPGdebug: set to %d\n", simple_debug);
+
+	ecpg_log("ECPGdebug: set to %d\n", simple_debug);
 
 #ifdef ENABLE_THREAD_SAFETY
 	pthread_mutex_unlock(&debug_init_mutex);
@@ -236,39 +251,54 @@ ECPGdebug(int n, FILE *dbgs)
 }
 
 void
-ECPGlog(const char *format,...)
+ecpg_log(const char *format,...)
 {
 	va_list		ap;
+	struct sqlca_t *sqlca = ECPGget_sqlca();
+	const char *intl_format;
+	int			bufsize;
+	char	   *fmt;
+
+	if (!simple_debug)
+		return;
+
+	/* internationalize the error message string */
+	intl_format = ecpg_gettext(format);
+
+	/*
+	 * Insert PID into the format, unless ecpg_internal_regression_mode is set
+	 * (regression tests want unchanging output).
+	 */
+	bufsize = strlen(intl_format) + 100;
+	fmt = (char *) malloc(bufsize);
+	if (fmt == NULL)
+		return;
+
+	if (ecpg_internal_regression_mode)
+		snprintf(fmt, bufsize, "[NO_PID]: %s", intl_format);
+	else
+		snprintf(fmt, bufsize, "[%d]: %s", (int) getpid(), intl_format);
 
 #ifdef ENABLE_THREAD_SAFETY
 	pthread_mutex_lock(&debug_mutex);
 #endif
 
-	if (simple_debug)
-	{
-		char	   *f = (char *) malloc(strlen(format) + 100);
+	va_start(ap, format);
+	vfprintf(debugstream, fmt, ap);
+	va_end(ap);
 
-		if (f == NULL)
-		{
-#ifdef ENABLE_THREAD_SAFETY
-			pthread_mutex_unlock(&debug_mutex);
-#endif
-			return;
-		}
+	/* dump out internal sqlca variables */
+	if (ecpg_internal_regression_mode)
+		fprintf(debugstream, "[NO_PID]: sqlca: code: %ld, state: %s\n",
+				sqlca->sqlcode, sqlca->sqlstate);
 
-		sprintf(f, "[%d]: %s", (int) getpid(), format);
-
-		va_start(ap, format);
-		vfprintf(debugstream, f, ap);
-		va_end(ap);
-		fflush(debugstream);
-
-		ECPGfree(f);
-	}
+	fflush(debugstream);
 
 #ifdef ENABLE_THREAD_SAFETY
 	pthread_mutex_unlock(&debug_mutex);
 #endif
+
+	free(fmt);
 }
 
 void
@@ -278,6 +308,7 @@ ECPGset_noind_null(enum ECPGttype type, void *ptr)
 	{
 		case ECPGt_char:
 		case ECPGt_unsigned_char:
+		case ECPGt_string:
 			*((char *) ptr) = '\0';
 			break;
 		case ECPGt_short:
@@ -293,12 +324,12 @@ ECPGset_noind_null(enum ECPGttype type, void *ptr)
 		case ECPGt_date:
 			*((long *) ptr) = LONG_MIN;
 			break;
-#ifdef HAVE_LONG_LONG_INT_64
+#ifdef HAVE_LONG_LONG_INT
 		case ECPGt_long_long:
 		case ECPGt_unsigned_long_long:
 			*((long long *) ptr) = LONG_LONG_MIN;
 			break;
-#endif   /* HAVE_LONG_LONG_INT_64 */
+#endif   /* HAVE_LONG_LONG_INT */
 		case ECPGt_float:
 			memset((char *) ptr, 0xff, sizeof(float));
 			break;
@@ -311,11 +342,11 @@ ECPGset_noind_null(enum ECPGttype type, void *ptr)
 			break;
 		case ECPGt_decimal:
 			memset((char *) ptr, 0, sizeof(decimal));
-			((decimal *) ptr)->sign = NUMERIC_NAN;
+			((decimal *) ptr)->sign = NUMERIC_NULL;
 			break;
 		case ECPGt_numeric:
 			memset((char *) ptr, 0, sizeof(numeric));
-			((numeric *) ptr)->sign = NUMERIC_NAN;
+			((numeric *) ptr)->sign = NUMERIC_NULL;
 			break;
 		case ECPGt_interval:
 			memset((char *) ptr, 0xff, sizeof(interval));
@@ -331,10 +362,11 @@ ECPGset_noind_null(enum ECPGttype type, void *ptr)
 static bool
 _check(unsigned char *ptr, int length)
 {
-	for (; ptr[--length] == 0xff && length >= 0; length--);
-	if (length < 0)
-		return true;
-	return false;
+	for (length--; length >= 0; length--)
+		if (ptr[length] != 0xff)
+			return false;
+
+	return true;
 }
 
 bool
@@ -344,6 +376,7 @@ ECPGis_noind_null(enum ECPGttype type, void *ptr)
 	{
 		case ECPGt_char:
 		case ECPGt_unsigned_char:
+		case ECPGt_string:
 			if (*((char *) ptr) == '\0')
 				return true;
 			break;
@@ -363,13 +396,13 @@ ECPGis_noind_null(enum ECPGttype type, void *ptr)
 			if (*((long *) ptr) == LONG_MIN)
 				return true;
 			break;
-#ifdef HAVE_LONG_LONG_INT_64
+#ifdef HAVE_LONG_LONG_INT
 		case ECPGt_long_long:
 		case ECPGt_unsigned_long_long:
 			if (*((long long *) ptr) == LONG_LONG_MIN)
 				return true;
 			break;
-#endif   /* HAVE_LONG_LONG_INT_64 */
+#endif   /* HAVE_LONG_LONG_INT */
 		case ECPGt_float:
 			return (_check(ptr, sizeof(float)));
 			break;
@@ -381,11 +414,11 @@ ECPGis_noind_null(enum ECPGttype type, void *ptr)
 				return true;
 			break;
 		case ECPGt_decimal:
-			if (((decimal *) ptr)->sign == NUMERIC_NAN)
+			if (((decimal *) ptr)->sign == NUMERIC_NULL)
 				return true;
 			break;
 		case ECPGt_numeric:
-			if (((numeric *) ptr)->sign == NUMERIC_NAN)
+			if (((numeric *) ptr)->sign == NUMERIC_NULL)
 				return true;
 			break;
 		case ECPGt_interval:
@@ -400,3 +433,125 @@ ECPGis_noind_null(enum ECPGttype type, void *ptr)
 
 	return false;
 }
+
+#ifdef WIN32
+#ifdef ENABLE_THREAD_SAFETY
+
+void
+win32_pthread_mutex(volatile pthread_mutex_t *mutex)
+{
+	if (mutex->handle == NULL)
+	{
+		while (InterlockedExchange((LONG *) &mutex->initlock, 1) == 1)
+			Sleep(0);
+		if (mutex->handle == NULL)
+			mutex->handle = CreateMutex(NULL, FALSE, NULL);
+		InterlockedExchange((LONG *) &mutex->initlock, 0);
+	}
+}
+
+static pthread_mutex_t win32_pthread_once_lock = PTHREAD_MUTEX_INITIALIZER;
+
+void
+win32_pthread_once(volatile pthread_once_t *once, void (*fn) (void))
+{
+	if (!*once)
+	{
+		pthread_mutex_lock(&win32_pthread_once_lock);
+		if (!*once)
+		{
+			*once = true;
+			fn();
+		}
+		pthread_mutex_unlock(&win32_pthread_once_lock);
+	}
+}
+#endif   /* ENABLE_THREAD_SAFETY */
+#endif   /* WIN32 */
+
+#ifdef ENABLE_NLS
+
+char *
+ecpg_gettext(const char *msgid)
+{
+	static bool already_bound = false;
+
+	if (!already_bound)
+	{
+		/* dgettext() preserves errno, but bindtextdomain() doesn't */
+#ifdef WIN32
+		int			save_errno = GetLastError();
+#else
+		int			save_errno = errno;
+#endif
+		const char *ldir;
+
+		already_bound = true;
+		/* No relocatable lookup here because the binary could be anywhere */
+		ldir = getenv("PGLOCALEDIR");
+		if (!ldir)
+			ldir = LOCALEDIR;
+		bindtextdomain(PG_TEXTDOMAIN("ecpglib"), ldir);
+#ifdef WIN32
+		SetLastError(save_errno);
+#else
+		errno = save_errno;
+#endif
+	}
+
+	return dgettext(PG_TEXTDOMAIN("ecpg"), msgid);
+}
+#endif   /* ENABLE_NLS */
+
+static struct var_list
+{
+	int			number;
+	void	   *pointer;
+	struct var_list *next;
+}	*ivlist = NULL;
+
+void
+ECPGset_var(int number, void *pointer, int lineno)
+{
+	struct var_list *ptr;
+
+	for (ptr = ivlist; ptr != NULL; ptr = ptr->next)
+	{
+		if (ptr->number == number)
+		{
+			/* already known => just change pointer value */
+			ptr->pointer = pointer;
+			return;
+		}
+	}
+
+	/* a new one has to be added */
+	ptr = (struct var_list *) calloc(1L, sizeof(struct var_list));
+	if (!ptr)
+	{
+		struct sqlca_t *sqlca = ECPGget_sqlca();
+
+		sqlca->sqlcode = ECPG_OUT_OF_MEMORY;
+		strncpy(sqlca->sqlstate, "YE001", sizeof(sqlca->sqlstate));
+		snprintf(sqlca->sqlerrm.sqlerrmc, sizeof(sqlca->sqlerrm.sqlerrmc), "out of memory on line %d", lineno);
+		sqlca->sqlerrm.sqlerrml = strlen(sqlca->sqlerrm.sqlerrmc);
+		/* free all memory we have allocated for the user */
+		ECPGfree_auto_mem();
+	}
+	else
+	{
+		ptr->number = number;
+		ptr->pointer = pointer;
+		ptr->next = ivlist;
+		ivlist = ptr;
+	}
+}
+
+void *
+ECPGget_var(int number)
+{
+	struct var_list *ptr;
+
+	for (ptr = ivlist; ptr != NULL && ptr->number != number; ptr = ptr->next);
+	return (ptr) ? ptr->pointer : NULL;
+}
