@@ -1,22 +1,20 @@
 /*-------------------------------------------------------------------------
  *
- * getusage.c
- *	  64-bit versions of fseeko/ftello()
+ * getrusage.c
+ *	  get information about resource utilisation
  *
- * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2010, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/port/getrusage.c,v 1.8 2004/12/31 22:03:53 pgsql Exp $
+ *	  $PostgreSQL: pgsql/src/port/getrusage.c,v 1.18 2010/01/02 16:58:13 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
 
-#include <stdio.h>
-#include <errno.h>
-
 #include "c.h"
+
 #include "rusagestub.h"
 
 /* This code works on:
@@ -37,13 +35,20 @@ getrusage(int who, struct rusage * rusage)
 {
 #ifdef WIN32
 
-	FILETIME starttime;
-	FILETIME exittime;
-	FILETIME kerneltime;
-	FILETIME usertime;
+	FILETIME	starttime;
+	FILETIME	exittime;
+	FILETIME	kerneltime;
+	FILETIME	usertime;
 	ULARGE_INTEGER li;
 
-	if (rusage == (struct rusage *)NULL)
+	if (who != RUSAGE_SELF)
+	{
+		/* Only RUSAGE_SELF is supported in this implementation for now */
+		errno = EINVAL;
+		return -1;
+	}
+
+	if (rusage == (struct rusage *) NULL)
 	{
 		errno = EFAULT;
 		return -1;
@@ -58,16 +63,15 @@ getrusage(int who, struct rusage * rusage)
 
 	/* Convert FILETIMEs (0.1 us) to struct timeval */
 	memcpy(&li, &kerneltime, sizeof(FILETIME));
-	li.QuadPart /= 10L; /* Convert to microseconds */
-	rusage->ru_stime.tv_sec  = li.QuadPart / 1000000L;
+	li.QuadPart /= 10L;			/* Convert to microseconds */
+	rusage->ru_stime.tv_sec = li.QuadPart / 1000000L;
 	rusage->ru_stime.tv_usec = li.QuadPart % 1000000L;
 
 	memcpy(&li, &usertime, sizeof(FILETIME));
-	li.QuadPart /= 10L; /* Convert to microseconds */
-	rusage->ru_utime.tv_sec  = li.QuadPart / 1000000L;
+	li.QuadPart /= 10L;			/* Convert to microseconds */
+	rusage->ru_utime.tv_sec = li.QuadPart / 1000000L;
 	rusage->ru_utime.tv_usec = li.QuadPart % 1000000L;
-
-#else /* all but WIN32 */
+#else							/* all but WIN32 */
 
 	struct tms	tms;
 	int			tick_rate = CLK_TCK;	/* ticks per second */
@@ -104,8 +108,7 @@ getrusage(int who, struct rusage * rusage)
 	rusage->ru_utime.tv_usec = TICK_TO_USEC(u, tick_rate);
 	rusage->ru_stime.tv_sec = TICK_TO_SEC(s, tick_rate);
 	rusage->ru_stime.tv_usec = TICK_TO_USEC(u, tick_rate);
-
-#endif /* WIN32 */
+#endif   /* WIN32 */
 
 	return 0;
 }
